#!/opt/local/bin/lua

require "List"
require "Test"

l = List.new()
eq(tostring(l), "#<Class List>")

-- basic tests
l:push(1)
l:push(2)
eq(l:pop(), 2)
eq(l:pop(), 1)

l:push(1)
l:unshift(2)
eq(l:size() , 2)
eq(l:first(), 2)
eq(l:last() , 1)
eq(l:shift(), 2)
eq(l:shift(), 1)

eq(l:size(), 0)
l:push(1)
eq(l:size(), 1)
l:clear()
eq(l:size(), 0)

-- multi-obj tests
l1 = List.new()
l1:push(1)
l:push(1)
l1:push(2)
l:push(2)
eq(l1:pop(), 2)
eq(l:pop() , 2)
eq(l1:pop(), 1)
eq(l:pop() , 1)

-- init with values
l2 = List.new(1, 2, 3, 4, 5)
eq(l2:pop(), 5)
eq(l2:pop(), 4)
eq(l2:shift(), 1)
eq(l2:shift(), 2)
eq(l2:pop(), 3)

-- last first
l3 = List.new(1, 2, 3, 4, 5)
eq(l3:last(3):size(), 3)
eq(l3:last(2):size(), 2)
eq(l3:last(1):size(), 1)

eq(l3:last(3):last(), 5)
eq(l3:last(3):first(), 3)

eq(l3:first(3):size(), 3)
eq(l3:first(2):size(), 2)
eq(l3:first(1):size(), 1)

eq(l3:first(3):last(), 3)
eq(l3:first(3):first(), 1)

-- join
l4 = List.new(1, 2, 3)
eq(l4:join(), "1, 2, 3")
eq(l4:join(" "), "1 2 3")
eq(List.new():join(" "), "")

-- + __add meta method
l5 = List.new(1, 2)
l6 = List.new(3, 4)
l7 = l5 + l6
eq(l7:join(), "1, 2, 3, 4")

-- - __sub meta method
l8 = List.new(1, 2, 1, 3, 1, 4, 1, 5) - List.new(2, 3, 4, 5)
eq(l8:join(), "1, 1, 1, 1")
l9 = List.new(1, 2, 1, 3, 1, 4, 1, 5) - List.new(1, 2, 3, 4, 5)
print(l9:join())
eq(l9:join(), "")

-- map
l10 = List.new(1, 2, 3, 4):map(function (i)
	return i * i
end)
eq(l10:join(), "1, 4, 9, 16")

-- min max
l11 = List.new(2, 5, 1, 2)
eq(l11:min(), 1)
eq(l11:max(), 5)

l12 = List.new({a = 1}, {a = 5}, {a = 3}, {a = 4})
eq(l12:min(function (i) return i.a end).a, 1)
eq(l12:max(function (i) return i.a end).a, 5)

-- == __eq meta method

l13 = List.new(1, 2, 3)
l14 = List.new(1, 2, 3)
l15 = List.new(1, 2, 3, 4)
eq(l13 == l14, true)
eq(l13 == l15, false)

-- select
l16 = List.new(1, 2, 3, 4, 5)
eq(l16:select(function (i) return i > 3 end), List.new(4, 5))

-- each
l17 = List.new(1, 2, 3, 4, 5)
local t = 1
l17:each(function (i)
	eq(i, t)
	t = t + 1
end)

l17:eachWithIndex(function (item, index)
	eq(item, index)
end)

-- index
l18 = List.new(10, 20, 30)
eq(l18[1], 10)
eq(l18[2], 20)
eq(l18[3], 30)


-- sort
l19 = List.new(4, 3, 1, 2, 9, 6, 2)
eq(l19:sort():join(), "1, 2, 2, 3, 4, 6, 9")
l20 = List.new({foo = 1}, {foo = 5}, {foo = 3}, {foo = 4}, {foo = 2}, {foo = 2})
eq(l20:sort(function (a, b) return (a.foo < b.foo) end):map(function (i) return i.foo end):join(), "1, 2, 2, 3, 4, 5")
eq(l20:sortBy(function (i) return i.foo end):map(function (i) return i.foo end):join(), "1, 2, 2, 3, 4, 5")

-- empty
l21 = List.new(1)
eq(l21:isEmpty(), false)
l21:clear()
eq(l21:isEmpty(), true)

-- isInclude, delete, deleteAt
l22 = List.new(1, 2, 2, 3)
eq(l22:isInclude(1), true)
eq(l22:isInclude(2), true)
eq(l22:isInclude(4), false)
eq(l22:deleteAt(1), 1)
eq(l22:deleteAt(1), 2)
eq(l22:join(), "2, 3")

eq(l22:delete(2), 2)
eq(l22:delete(4), nil)
eq(l22:isInclude(2), false)
eq(l22:join(), "3")
